# include <glib.h>
# include <stdio.h>
# include <unistd.h>
# include <string.h>
# include <sys/types.h>
# include <sys/wait.h>
# include <string.h>
# include <errno.h>
# include "peer.h"
# include "conf.h"
# include "util.h"

extern char **environ;

static void fw_exec_add_env ( gchar *key, gchar *val, GPtrArray *env ) {
    gchar *p = g_strdup_printf( "%s=%s", key, val );
    g_ptr_array_add( env, p );
}

static int fw_exec( gchar *action, GHashTable *conf, peer *p ) {
    GHashTable *data;
    GPtrArray *env;
    gchar *cmd, **arg, **n;

    data = g_hash_dup( conf );
    //
    // Than add specifics about this particular client, if any
    if (p != NULL) {
	g_hash_set( data, "IP",    p->ip );
	g_hash_set( data, "MAC",   p->hw );
	g_hash_set( data, "Class", "Public" );
    }

    cmd = conf_string( conf, action );
    cmd = parse_template( cmd, data );
    g_warning("Got command %s from action %s", cmd, action );
    arg = g_strsplit( cmd, " ", 0 );

    // prime the environment with our existing environment
    env = g_ptr_array_new();
    for ( n = environ; *n != NULL; n++ )
	g_ptr_array_add( env, *n );

    // Then add everything from the conf file
    g_hash_table_foreach( data, (GHFunc) fw_exec_add_env, env );

    /* We're not cleaning up memory references because 
     * hopefully the exec won't fail... */
    execve( arg[0], (char **)arg, (char **)env->pdata );
    g_error( "execve %s failed: %m", cmd ); // Shouldn't happen.
    return -1;
}

int fw_perform( gchar *action, GHashTable *conf, peer *p ) {
    pid_t pid, r = 0;
    guint status = 0, retval = 0;

    pid = fork();
    if (pid == -1)
	g_error( "Can't fork: %m" );
    
    if (! pid)
	fw_exec( action, conf, p );

    while (r != pid) {
	r = waitpid( pid, &status, 0 );
	if (r == -1 && errno != EINTR) {
	    g_warning( "waitpid failed: %m" );
	    break;
	}
    }

    if (WIFEXITED(status)) {
	retval = WEXITSTATUS(status);
	if (retval)
	    g_warning( "%s of peer %s returned %d", action, p->ip, retval );
    } else if (WIFSIGNALED(status)) {
	retval = WTERMSIG(status);
	g_warning( "%s of peer %s died from signal %d", action, p->ip, retval );
    }

    return retval;
}

int fw_init ( GHashTable *conf ) {
    return fw_perform( "ResetCmd", conf, NULL );
}

gchar *peer_arp( peer *p ) {
    gchar ip[16], hw[18];
    FILE *arp;

    g_assert( p != NULL );

    arp = fopen( "/proc/net/arp", "r" );
    if ( arp == NULL )
	g_error( "Can't open /proc/net/arp: %m" );
   
    fscanf(arp, "%*s %*s %*s %*s %*s %*s %*s %*s %*s"); // Skip first line 
    while (fscanf( arp, "%15s %*s %*s %17s %*s %*s\n", ip, hw ) != EOF)  {
	if ( strncmp( p->ip, ip, sizeof(p->ip) ) == 0 ) {
	    g_strncpy( p->hw, hw, sizeof(p->hw) );
	    break;
	}
    }

    fclose( arp );
    return ( p->hw[0] != '\0' ? p->hw : NULL );
}

