# include <glib.h>
# include <string.h>
# include <unistd.h>
# include "http.h"
# include "conf.h"
# include "util.h"

void accept_peer ( http_request *h );

/************* Capture and splash *************/

void capture_peer ( http_request *h ) {
    gchar *host   = HEADER("Host"); 
    gchar *orig, *redir, *dest;
    
    if ( host != NULL ) {
	orig = g_strdup_printf( "http://%s%s", HEADER("Host"), h->uri );
    } else {
	orig = CONF("HomePage");
    }
	
    redir  = url_encode( orig );
    dest   = g_strdup_printf( "http://%s:%s/?redirect=%s",
	h->sock_ip, CONF("GatewayPort"), redir ); 

    http_add_header ( h, "Location", dest );
    http_send_header( h, 302, "Moved" );

    g_free( orig  );
    g_free( redir );
    g_free( dest  );
}

void splash_peer ( http_request *h ) {
    GHashTable *data;
    gchar *path, *file, *form, *action;
    ssize_t n;
    GIOError r;
   
    action = g_strdup_printf( "http://%s:%s/", h->sock_ip, CONF("GatewayPort") ); 
    data = g_hash_dup( nocat_conf );
    g_hash_merge( data, h->query );
    g_hash_set( data, "action", action );

    path = http_fix_path( CONF("SplashForm"), CONF("DocumentRoot") );
    file = load_file( path );
  
    form = parse_template( file, data );
    n = strlen(form);

    http_add_header( h, "Content-Type", http_mime_type(path) );
    http_send_header( h, 200, "OK" );
    r = g_io_channel_write( h->sock, form, n, &n );
    if ( r != G_IO_ERROR_NONE )
	g_warning( "Splashing %s failed: %m", h->peer_ip );

    g_hash_free( data );
    g_free( action );
    g_free( form );
    g_free( file );
    g_free( path );
}

void handle_request( http_request *h ) {
    gchar *hostname = HEADER("Host");
    gchar *sockname = g_strdup_printf( "%s:%s", 
	    h->sock_ip, CONF("GatewayPort") ); 

    g_assert( sockname != NULL );
    // g_message( "got sockname: '%s'", sockname );
    g_assert( hostname != NULL );
    // g_message( "got hostname: '%s'", hostname );

    // g_message( "%s == %s = %d", hostname, sockname, n );
    if (hostname == NULL || strcmp( hostname, sockname ) != 0) {
	capture_peer(h);
    } else if (strcmp( h->uri, "/" ) == 0) {
	if ( QUERY("mode_login") != NULL || QUERY("mode_login.x") != NULL ) {
	    accept_peer(h);
	} else if ( QUERY("redirect") != NULL ) {
	    splash_peer(h);
	} else {
	    capture_peer(h);
	}
    } else {
	http_serve_file( h, CONF("DocumentRoot") );
    }

    g_free( sockname );
}

