# include <glib.h>
# include <time.h>
# include <string.h>
# include "peer.h"
# include "firewall.h"
# include "conf.h"

peer *peer_new ( const gchar *ip ) {
    peer *p = g_new0( peer, 1 );
    g_assert( p != NULL );
    g_assert( ip != NULL );
    // Set IP address.
    strncpy( p->ip, ip, sizeof(p->ip) );
    // Set MAC address.
    peer_arp( p );
    // Set connection time.
    p->connected = time( NULL );
    return p;
}

void peer_free ( peer *p ) {
    g_assert( p != NULL );
    if (p->request != NULL)
	g_free( p->request );
    g_free(p);
}

int peer_permit ( GHashTable *conf, peer *p ) {
    g_assert( p != NULL );
    if (p->status != PEER_ACCEPT) {
	if (fw_perform( "PermitCmd", conf, p ) == 0) {
	    p->status = PEER_ACCEPT;
	} else {
	    return -1;
	}
    }
    p->expire = time(NULL) + conf_int( conf, "LoginTimeout" );
    return 0;
}

int peer_deny ( GHashTable *conf, peer *p ) {
    g_assert( p != NULL );
    if (p->status != PEER_DENY) {
	if (fw_perform( "DenyCmd", conf, p ) == 0) {
	    p->status = PEER_DENY;
	} else {
	    return -1;
	}
    }
    return 0;
}

